@@ -17,6 +17,7 @@ import enum
 import math
 import operator
 import random
+import re
 import string
 import sys
 import threading
@@ -25,6 +26,7 @@ from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
 from inspect import Parameter, getfullargspec, isabstract, isclass, signature
+from types import FunctionType
 from typing import (
     Any,
     AnyStr,
@@ -48,7 +50,6 @@ from uuid import UUID
 
 import attr
 
-from hypothesis._settings import note_deprecation
 from hypothesis.control import cleanup, note, reject
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.cache import LRUReusedCache
@@ -72,7 +73,6 @@ from hypothesis.internal.floats import (
 )
 from hypothesis.internal.reflection import (
     define_function_signature,
-    deprecated_posargs,
     get_pretty_function_description,
     is_typed_named_tuple,
     nicerepr,
@@ -296,11 +296,55 @@ def none() -> SearchStrategy[None]:
     return just(None)
 
 
+T4 = TypeVar("T4")
+T5 = TypeVar("T5")
+
+
 @overload
 def one_of(args: Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]:
     raise NotImplementedError
 
 
+@overload  # noqa: F811
+def one_of(a1: SearchStrategy[Ex]) -> SearchStrategy[Ex]:
+    raise NotImplementedError
+
+
+@overload  # noqa: F811
+def one_of(
+    a1: SearchStrategy[Ex], a2: SearchStrategy[K]
+) -> SearchStrategy[Union[Ex, K]]:
+    raise NotImplementedError
+
+
+@overload  # noqa: F811
+def one_of(
+    a1: SearchStrategy[Ex], a2: SearchStrategy[K], a3: SearchStrategy[V]
+) -> SearchStrategy[Union[Ex, K, V]]:
+    raise NotImplementedError
+
+
+@overload  # noqa: F811
+def one_of(
+    a1: SearchStrategy[Ex],
+    a2: SearchStrategy[K],
+    a3: SearchStrategy[V],
+    a4: SearchStrategy[T4],
+) -> SearchStrategy[Union[Ex, K, V, T4]]:
+    raise NotImplementedError
+
+
+@overload  # noqa: F811
+def one_of(
+    a1: SearchStrategy[Ex],
+    a2: SearchStrategy[K],
+    a3: SearchStrategy[V],
+    a4: SearchStrategy[T4],
+    a5: SearchStrategy[T5],
+) -> SearchStrategy[Union[Ex, K, V, T4, T5]]:
+    raise NotImplementedError
+
+
 @overload  # noqa: F811
 def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:
     raise NotImplementedError
@@ -423,7 +467,6 @@ def booleans() -> SearchStrategy[bool]:
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def floats(
     min_value: Optional[Real] = None,
     max_value: Optional[Real] = None,
@@ -495,19 +538,19 @@ def floats(
 
     if min_value != min_arg:
         raise InvalidArgument(
-            "min_value=%r cannot be exactly represented as a float of width "
-            "%d - use min_value=%r instead." % (min_arg, width, min_value)
+            f"min_value={min_arg!r} cannot be exactly represented as a float "
+            f"of width {width} - use min_value={min_value!r} instead."
         )
     if max_value != max_arg:
         raise InvalidArgument(
-            "max_value=%r cannot be exactly represented as a float of width "
-            "%d - use max_value=%r instead" % (max_arg, width, max_value)
+            f"max_value={max_arg!r} cannot be exactly represented as a float "
+            f"of width {width} - use max_value={max_value!r} instead."
         )
 
     if exclude_min and (min_value is None or min_value == math.inf):
-        raise InvalidArgument("Cannot exclude min_value=%r" % (min_value,))
+        raise InvalidArgument(f"Cannot exclude min_value={min_value!r}")
     if exclude_max and (max_value is None or max_value == -math.inf):
-        raise InvalidArgument("Cannot exclude max_value=%r" % (max_value,))
+        raise InvalidArgument(f"Cannot exclude max_value={max_value!r}")
 
     if min_value is not None and (
         exclude_min or (min_arg is not None and min_value < min_arg)
@@ -550,7 +593,7 @@ def floats(
             "and max_value=%r" % (width, min_arg, max_arg)
         )
         if exclude_min or exclude_max:
-            msg += ", exclude_min=%r and exclude_max=%r" % (exclude_min, exclude_max)
+            msg += f", exclude_min={exclude_min!r} and exclude_max={exclude_max!r}"
         raise InvalidArgument(msg)
 
     if allow_infinity is None:
@@ -682,9 +725,9 @@ def sampled_from(elements):
     if len(values) == 1:
         return just(values[0])
     if isinstance(elements, type) and issubclass(elements, enum.Enum):
-        repr_ = "sampled_from(%s.%s)" % (elements.__module__, elements.__name__)
+        repr_ = f"sampled_from({elements.__module__}.{elements.__name__})"
     else:
-        repr_ = "sampled_from(%r)" % (elements,)
+        repr_ = f"sampled_from({elements!r})"
     if isclass(elements) and issubclass(elements, enum.Flag):
         # Combinations of enum.Flag members are also members.  We generate
         # these dynamically, because static allocation takes O(2^n) memory.
@@ -696,9 +739,12 @@ def sampled_from(elements):
     return SampledFromStrategy(values, repr_)
 
 
+def identity(x):
+    return x
+
+
 @cacheable
 @defines_strategy()
-@deprecated_posargs
 def lists(
     elements: SearchStrategy[Ex],
     *,
@@ -743,9 +789,7 @@ def lists(
                 "(you probably only want to set unique_by)"
             )
         else:
-
-            def unique_by(x):
-                return x
+            unique_by = identity
 
     if max_size == 0:
         return builds(list)
@@ -763,13 +807,36 @@ def lists(
                 raise InvalidArgument("unique_by[%i]=%r is not a callable" % (i, f))
         # Note that lazy strategies automatically unwrap when passed to a defines_strategy
         # function.
+        tuple_suffixes = None
+        if (
+            # We're generating a list of tuples unique by the first element, perhaps
+            # via st.dictionaries(), and this will be more efficient if we rearrange
+            # our strategy somewhat to draw the first element then draw add the rest.
+            isinstance(elements, TupleStrategy)
+            and len(elements.element_strategies) >= 1
+            and len(unique_by) == 1
+            and (
+                # Introspection for either `itemgetter(0)`, or `lambda x: x[0]`
+                isinstance(unique_by[0], operator.itemgetter)  # type: ignore
+                and repr(unique_by[0]) == "operator.itemgetter(0)"
+                or isinstance(unique_by[0], FunctionType)
+                and re.fullmatch(
+                    get_pretty_function_description(unique_by[0]),
+                    r"lambda ([a-z]+): \1\[0\]",
+                )
+            )
+        ):
+            unique_by = (identity,)
+            tuple_suffixes = TupleStrategy(elements.element_strategies[1:])
+            elements = elements.element_strategies[0]
+
         if isinstance(elements, SampledFromStrategy):
             element_count = len(elements.elements)
             if min_size > element_count:
                 raise InvalidArgument(
-                    "Cannot create a collection of min_size=%r unique elements with "
-                    "values drawn from only %d distinct elements"
-                    % (min_size, element_count)
+                    f"Cannot create a collection of min_size={min_size!r} unique "
+                    f"elements with values drawn from only {element_count} distinct "
+                    "elements"
                 )
 
             if max_size is not None:
@@ -778,18 +845,25 @@ def lists(
                 max_size = element_count
 
             return UniqueSampledListStrategy(
-                elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
+                elements=elements,
+                max_size=max_size,
+                min_size=min_size,
+                keys=unique_by,
+                tuple_suffixes=tuple_suffixes,
             )
 
         return UniqueListStrategy(
-            elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
+            elements=elements,
+            max_size=max_size,
+            min_size=min_size,
+            keys=unique_by,
+            tuple_suffixes=tuple_suffixes,
         )
     return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
 
 @cacheable
 @defines_strategy()
-@deprecated_posargs
 def sets(
     elements: SearchStrategy[Ex],
     *,
@@ -812,7 +886,6 @@ def sets(
 
 @cacheable
 @defines_strategy()
-@deprecated_posargs
 def frozensets(
     elements: SearchStrategy[Ex],
     *,
@@ -842,7 +915,6 @@ class PrettyIter:
 
 
 @defines_strategy()
-@deprecated_posargs
 def iterables(
     elements: SearchStrategy[Ex],
     *,
@@ -889,11 +961,11 @@ def fixed_dictionaries(
     """
     check_type(dict, mapping, "mapping")
     for k, v in mapping.items():
-        check_strategy(v, "mapping[%r]" % (k,))
+        check_strategy(v, f"mapping[{k!r}]")
     if optional is not None:
         check_type(dict, optional, "optional")
         for k, v in optional.items():
-            check_strategy(v, "optional[%r]" % (k,))
+            check_strategy(v, f"optional[{k!r}]")
         if type(mapping) != type(optional):
             raise InvalidArgument(
                 "Got arguments of different types: mapping=%s, optional=%s"
@@ -910,7 +982,6 @@ def fixed_dictionaries(
 
 @cacheable
 @defines_strategy()
-@deprecated_posargs
 def dictionaries(
     keys: SearchStrategy[Ex],
     values: SearchStrategy[T],
@@ -940,13 +1011,12 @@ def dictionaries(
         tuples(keys, values),
         min_size=min_size,
         max_size=max_size,
-        unique_by=lambda x: x[0],
+        unique_by=operator.itemgetter(0),
     ).map(dict_class)
 
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def characters(
     *,
     whitelist_categories: Optional[Sequence[str]] = None,
@@ -1004,9 +1074,9 @@ def characters(
     ):
         raise InvalidArgument(
             "Nothing is excluded by other arguments, so passing only "
-            "whitelist_characters=%(chars)r would have no effect.  Also pass "
-            "whitelist_categories=(), or use sampled_from(%(chars)r) instead."
-            % {"chars": whitelist_characters}
+            f"whitelist_characters={whitelist_characters!r} would have no effect.  "
+            "Also pass whitelist_categories=(), or use "
+            f"sampled_from({whitelist_characters!r}) instead."
         )
     blacklist_characters = blacklist_characters or ""
     whitelist_characters = whitelist_characters or ""
@@ -1053,7 +1123,6 @@ def characters(
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def text(
     alphabet: Union[Sequence[str], SearchStrategy[str]] = characters(
         blacklist_categories=("Cs",)
@@ -1109,7 +1178,6 @@ def text(
 
 @cacheable
 @defines_strategy()
-@deprecated_posargs
 def from_regex(
     regex: Union[AnyStr, Pattern[AnyStr]], *, fullmatch: bool = False
 ) -> SearchStrategy[AnyStr]:
@@ -1150,7 +1218,6 @@ def from_regex(
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def binary(
     *,
     min_size: int = 0,
@@ -1177,7 +1244,7 @@ def binary(
 def randoms(
     *,
     note_method_calls: bool = False,
-    use_true_random: Optional[bool] = None,
+    use_true_random: bool = False,
 ) -> SearchStrategy[random.Random]:
     """Generates instances of ``random.Random``. The generated Random instances
     are of a special HypothesisRandom subclass.
@@ -1193,15 +1260,8 @@ def randoms(
       flag should only be set to True when your code relies on the distribution
       of values for correctness.
     """
-    if use_true_random is None:
-        note_deprecation(
-            """Defaulting to old behaviour of use_true_random=True. If you want
-            to keep that behaviour, set use_true_random=True explicitly. If you
-            want the new behaviour (which will become the default in future),
-            set use_true_random=False.""",
-            since="2020-06-30",
-        )
-        use_true_random = True
+    check_type(bool, note_method_calls, "note_method_calls")
+    check_type(bool, use_true_random, "use_true_random")
 
     from hypothesis.strategies._internal.random import RandomStrategy
 
@@ -1215,7 +1275,7 @@ class RandomSeeder:
         self.seed = seed
 
     def __repr__(self):
-        return "RandomSeeder(%r)" % (self.seed,)
+        return f"RandomSeeder({self.seed!r})"
 
 
 class RandomModule(SearchStrategy):
@@ -1246,6 +1306,36 @@ def random_module() -> SearchStrategy[RandomSeeder]:
     return shared(RandomModule(), key="hypothesis.strategies.random_module()")
 
 
+class BuildsStrategy(SearchStrategy):
+    def __init__(self, target, args, kwargs):
+        self.target = target
+        self.args = args
+        self.kwargs = kwargs
+
+    def do_draw(self, data):
+        try:
+            return self.target(
+                *[data.draw(a) for a in self.args],
+                **{k: data.draw(v) for k, v in self.kwargs.items()},
+            )
+        except TypeError as err:
+            if (
+                isinstance(self.target, type)
+                and issubclass(self.target, enum.Enum)
+                and not (self.args or self.kwargs)
+            ):
+                name = self.target.__module__ + "." + self.target.__qualname__
+                raise InvalidArgument(
+                    f"Calling {name} with no arguments raised an error - "
+                    f"try using sampled_from({name}) instead of builds({name})"
+                ) from err
+            raise
+
+    def validate(self):
+        tuples(*self.args).validate()
+        fixed_dictionaries(self.kwargs).validate()
+
+
 # The ideal signature builds(target, /, *args, **kwargs) is unfortunately a
 # SyntaxError before Python 3.8 so we emulate it with manual argument unpacking.
 # Note that for the benefit of documentation and introspection tools, we set the
@@ -1313,27 +1403,9 @@ def builds(
         for kw in set(hints) & (required | to_infer):
             kwargs[kw] = from_type(hints[kw])
 
-    def build_target(value):
-        args, kwargs = value
-        try:
-            return target(*args, **kwargs)
-        except TypeError as err:
-            if (
-                isinstance(target, type)
-                and issubclass(target, enum.Enum)
-                and not (args or kwargs)
-            ):
-                name = target.__module__ + "." + target.__qualname__
-                raise InvalidArgument(
-                    f"Calling {name} with no arguments raised an error - "
-                    f"try using sampled_from({name}) instead of builds({name})"
-                ) from err
-            raise
-
     # Mypy doesn't realise that `infer` is gone from kwargs now
     # and thinks that target and args have the same (union) type.
-    args_kwargs = tuples(tuples(*args), fixed_dictionaries(kwargs))  # type: ignore
-    return args_kwargs.map(build_target)
+    return BuildsStrategy(target, args, kwargs)
 
 
 if sys.version_info[:2] >= (3, 8):  # pragma: no cover
@@ -1394,7 +1466,8 @@ def from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
 
         def everything_except(excluded_types):
             return (
-                from_type(type).flatmap(from_type)
+                from_type(type)
+                .flatmap(from_type)
                 .filter(lambda x: not isinstance(x, excluded_types))
             )
 
@@ -1417,7 +1490,7 @@ def from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             lambda thing: deferred(lambda: _from_type(thing)),
             (thing,),
             {},
-            force_repr="from_type(%r)" % (thing,),
+            force_repr=f"from_type({thing!r})",
         )
 
 
@@ -1465,7 +1538,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
                 % (thing, nicerepr(strat_or_callable), strategy)
             )
         if strategy.is_empty:
-            raise ResolutionFailed("Error: %r resolved to an empty strategy" % (thing,))
+            raise ResolutionFailed(f"Error: {thing!r} resolved to an empty strategy")
         return strategy
 
     if not isinstance(thing, type):
@@ -1481,7 +1554,22 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             args = sorted(thing.__args__, key=types.type_sorting_key)
             return one_of([from_type(t) for t in args])
     if not types.is_a_type(thing):
-        raise InvalidArgument("thing=%s must be a type" % (thing,))
+        # The implementation of typing_extensions.Literal under Python 3.6 is
+        # *very strange*.  Notably, `type(Literal[x]) != Literal` so we have to
+        # use the first form directly, and because it uses __values__ instead of
+        # __args__ we inline the relevant logic here until the end of life date.
+        if types.is_typing_literal(thing):  # pragma: no cover
+            assert sys.version_info[:2] == (3, 6)
+            args_dfs_stack = list(thing.__values__)  # type: ignore
+            literals = []
+            while args_dfs_stack:
+                arg = args_dfs_stack.pop()
+                if types.is_typing_literal(arg):
+                    args_dfs_stack.extend(reversed(arg.__values__))
+                else:
+                    literals.append(arg)
+            return sampled_from(literals)
+        raise InvalidArgument(f"thing={thing} must be a type")
     # Now that we know `thing` is a type, the first step is to check for an
     # explicitly registered strategy.  This is the best (and hopefully most
     # common) way to resolve a type to a strategy.  Note that the value in the
@@ -1542,9 +1630,29 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             "Could not resolve %r to a strategy; consider "
             "using register_type_strategy" % (thing,)
         )
-    # Finally, try to build an instance by calling the type object
+    # Finally, try to build an instance by calling the type object.  Unlike builds(),
+    # this block *does* try to infer strategies for arguments with default values.
+    # That's because of the semantic different; builds() -> "call this with ..."
+    # so we only infer when *not* doing so would be an error; from_type() -> "give
+    # me arbitrary instances" so the greater variety is acceptable.
+    # And if it's *too* varied, express your opinions with register_type_strategy()
     if not isabstract(thing):
-        return builds(thing)
+        try:
+            hints = get_type_hints(thing)
+            params = signature(thing).parameters
+        except Exception:
+            return builds(thing)
+        kwargs = {}
+        for k, p in params.items():
+            if (
+                k in hints
+                and k != "return"
+                and p.default is not Parameter.empty
+                and p.kind in (Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY)
+            ):
+                kwargs[k] = just(p.default) | _from_type(hints[k])
+        return builds(thing, **kwargs)
+    # And if it's an abstract type, we'll resolve to a union of subclasses instead.
     subclasses = thing.__subclasses__()
     if not subclasses:
         raise ResolutionFailed(
@@ -1556,7 +1664,6 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def fractions(
     min_value: Optional[Union[Real, str]] = None,
     max_value: Optional[Union[Real, str]] = None,
@@ -1658,15 +1765,14 @@ def _as_finite_decimal(
         if allow_infinity or allow_infinity is None:
             return None
         raise InvalidArgument(
-            "allow_infinity=%r, but %s=%r" % (allow_infinity, name, value)
+            f"allow_infinity={allow_infinity!r}, but {name}={value!r}"
         )
     # This could be infinity, quiet NaN, or signalling NaN
-    raise InvalidArgument("Invalid %s=%r" % (name, value))
+    raise InvalidArgument(f"Invalid {name}={value!r}")
 
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def decimals(
     min_value: Optional[Union[Real, str]] = None,
     max_value: Optional[Union[Real, str]] = None,
@@ -1727,8 +1833,8 @@ def decimals(
             max_num = floor(ctx(max_value).divide(max_value, factor))
         if min_num is not None and max_num is not None and min_num > max_num:
             raise InvalidArgument(
-                "There are no decimals with %d places between min_value=%r "
-                "and max_value=%r " % (places, min_value, max_value)
+                f"There are no decimals with {places} places between "
+                f"min_value={min_value!r} and max_value={max_value!r}"
             )
         strat = integers(min_num, max_num).map(int_to_decimal)
     else:
@@ -1754,7 +1860,6 @@ def decimals(
     return strat | (sampled_from(special) if special else nothing())
 
 
-@deprecated_posargs
 def recursive(
     base: SearchStrategy[Ex],
     extend: Callable[[SearchStrategy[Any]], SearchStrategy[T]],
@@ -1877,7 +1982,6 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
 
 @defines_strategy(force_reusable_values=True)
 @cacheable
-@deprecated_posargs
 def complex_numbers(
     *,
     min_magnitude: Real = 0,
@@ -1911,8 +2015,6 @@ def complex_numbers(
     check_valid_magnitude(min_magnitude, "min_magnitude")
     check_valid_magnitude(max_magnitude, "max_magnitude")
     check_valid_interval(min_magnitude, max_magnitude, "min_magnitude", "max_magnitude")
-    if min_magnitude is None:
-        min_magnitude = 0
     if max_magnitude == math.inf:
         max_magnitude = None
 
@@ -1963,7 +2065,6 @@ def complex_numbers(
     return constrained_complex()
 
 
-@deprecated_posargs
 def shared(
     base: SearchStrategy[Ex],
     *,
@@ -1990,7 +2091,6 @@ def shared(
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def uuids(*, version: Optional[int] = None) -> SearchStrategy[UUID]:
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
@@ -2035,7 +2135,6 @@ class RunnerStrategy(SearchStrategy):
 
 
 @defines_strategy(force_reusable_values=True)
-@deprecated_posargs
 def runner(*, default: Any = not_set) -> SearchStrategy[Any]:
     """A strategy for getting "the current test runner", whatever that may be.
     The exact meaning depends on the entry point, but it will usually be the
@@ -2069,9 +2168,9 @@ class DataObject:
         result = self.conjecture_data.draw(strategy)
         self.count += 1
         if label is not None:
-            note("Draw %d (%s): %r" % (self.count, label, result))
+            note(f"Draw {self.count} ({label}): {result!r}")
         else:
-            note("Draw %d: %r" % (self.count, result))
+            note(f"Draw {self.count}: {result!r}")
         return result
 
 
@@ -2158,25 +2257,12 @@ def register_type_strategy(
         raise InvalidArgument("strategy=%r must not be empty")
     elif types.has_type_arguments(custom_type):
         origin = getattr(custom_type, "__origin__", None)
-        if callable(strategy):
-            strategy_repr = get_pretty_function_description(strategy)
-        else:
-            strategy_repr = repr(strategy)
-        note_deprecation(
-            "Registering a generic type with arguments (%r) is deprecated, and "
-            "will be an error in future, because the resolution logic is badly "
-            "broken.  Instead, register a function for the origin type (%r) "
-            "which can inspect specific type objects and return a strategy.  "
-            "%s will be registered for any type arguments."
-            % (custom_type, origin, strategy_repr),
-            since="2020-08-17",
+        raise InvalidArgument(
+            f"Cannot register generic type {custom_type!r}, because it has type "
+            "arguments which would not be handled.  Instead, register a function "
+            f"for {origin!r} which can inspect specific type objects and return a "
+            "strategy."
         )
-        if origin in types._global_type_lookup:
-            raise InvalidArgument(
-                "Cannot register %r, because the without-arguments form %r already "
-                "has a registered strategy %s" % (custom_type, origin, strategy_repr)
-            )
-        custom_type = origin
 
     types._global_type_lookup[custom_type] = strategy
     from_type.__clear_cache()  # type: ignore
@@ -2236,7 +2322,6 @@ def emails() -> SearchStrategy[str]:
 
 
 @defines_strategy()
-@deprecated_posargs
 def functions(
     *,
     like: Callable[..., Any] = lambda: None,
